#include "common.hlsl"

//--------------------------------------------------------------------------------------
// Vertex Shader
v2f_t VertexFunction(vs_input_t input)
{
	v2f_t v2f = (v2f_t)0;

	// forward vertex input onto the next stage
	v2f.position = float4(input.position, 1.0f);
	v2f.color = input.color;
	v2f.uv = input.uv;

	float4 modelPos = float4(input.position, 1);
	float4 worldPos = mul(MODEL, modelPos);
	float4 cameraPos = mul(VIEW, worldPos);
	float4 clipPos = mul(PROJECTION, cameraPos);

	v2f.position = clipPos;

	return v2f;
}

// raster step
// float3 ndcPos = clipPos.xyz / clipPos.w;

//--------------------------------------------------------------------------------------
// Fragment Shader
// 
// SV_Target0 at the end means the float4 being returned
// is being drawn to the first bound color target.
float4 FragmentFunction(v2f_t input) : SV_Target0
{
	float2 uv = input.uv;

	float4 color = tDiffuse.Sample(sSampler, uv);
	
	clip(color.a - 0.01f);
	
	return color * input.color;

}