#include "common.hlsl"

//--------------------------------------------------------------------------------------
// Vertex Shader
v2f_t VertexFunction(vs_input_t input)
{
	v2f_t v2f = (v2f_t)0;

	// move the vertex through the spaces
	float4 local_pos = float4(input.position, 1.0f);               // passed in position is usually inferred to be "local position", ie, local to the object
	float4 world_pos = mul(MODEL, local_pos);               // world pos is the object moved to its place int he world by the model, not used yet
	float4 camera_pos = mul(VIEW, world_pos);
	float4 clip_pos = mul(PROJECTION, camera_pos);

	// normal is currently in model/local space
	float4 local_normal = float4(input.normal, 0);
	float4 world_normal = mul(MODEL, local_normal);

	float4 local_tangent = float4(input.tangent, 0);
	float4 world_tangent = mul(MODEL, local_tangent);

	float4 local_bitangent = float4(input.bitangent, 0);
	float4 world_bitangent = mul(MODEL, local_bitangent);

	world_normal = float4(cross(world_tangent.xyz, world_bitangent.xyz), 0);

	// tangent & bitangent

	v2f.position = clip_pos; // we want to output the clip position to raster (a perspective point)
	v2f.color = input.color;
	v2f.uv = input.uv;
	v2f.world_position = world_pos.xyz;
	v2f.world_normal = world_normal.xyz;
	v2f.world_tangent = world_tangent.xyz;
	v2f.world_bitangent = world_bitangent.xyz;

	return v2f;
}

//--------------------------------------------------------------------------------------
// Fragment Shader
// 
// SV_Target0 at the end means the float4 being returned
// is being drawn to the first bound color target.

float4 FragmentFunction(v2f_t input) : SV_Target0
{
    float3 normal = normalize(input.world_normal); 
    float3 tangent = normalize(input.world_tangent);
    float3 bitangent = normalize(input.world_bitangent); 
    
    return normalizedVectorToColor(normal);
}