#include "common.hlsl"

cbuffer projection_consts : register(b4)
{
	float4x4 PROJECTION_MATRIX; // actually world -> clip
    float3 PROJECTOR_POSITION; 
    float PROJECTOR_STRENGTH; 
} 

Texture2D <float4> tProjection  : register(t8);   // dissolve texture


//--------------------------------------------------------------------------------------
// Vertex Shader
v2f_t VertexFunction(vs_input_t input)
{
	v2f_t v2f = (v2f_t)0;

	// move the vertex through the spaces
	float4 local_pos = float4(input.position, 1.0f);               // passed in position is usually inferred to be "local position", ie, local to the object
	float4 world_pos = mul(MODEL, local_pos);               // world pos is the object moved to its place int he world by the model, not used yet
	float4 camera_pos = mul(VIEW, world_pos);
	float4 clip_pos = mul(PROJECTION, camera_pos);

	// normal is currently in model/local space
	float4 local_normal = float4(input.normal, 0);
	float4 world_normal = mul(MODEL, local_normal);

	float4 local_tangent = float4(input.tangent, 0);
	float4 world_tangent = mul(MODEL, local_tangent);

	float4 local_bitangent = float4(input.bitangent, 0);
	float4 world_bitangent = mul(MODEL, local_bitangent);

	// tangent & bitangent

	v2f.position = clip_pos; // we want to output the clip position to raster (a perspective point)
	v2f.color = input.color;
	v2f.uv = input.uv;
	v2f.world_position = world_pos.xyz;
	v2f.world_normal = world_normal.xyz;
	v2f.world_tangent = world_tangent.xyz;
	v2f.world_bitangent = world_bitangent.xyz;

	return v2f;
}

float4 FragmentFunction(v2f_t input) : SV_Target0
{
    float3 dir_to_eye = normalize(PROJECTOR_POSITION - input.world_position);
    float3 normal = normalize(input.world_normal);

    float4 clip_pos = mul(PROJECTION_MATRIX, float4(input.world_position, 1.0f));
    float3 ndc = clip_pos.xyz / clip_pos.w;

    float final_blend = step(0.01f, clip_pos.w);
    float dot3 = dot(normal, dir_to_eye);

    final_blend *= max(0.0f, dot3) * PROJECTOR_STRENGTH;

    float2 uv = (ndc.xy + float2(1.0f, 1.0f)) * 0.5f;
    
    float visible = step(0.0f, uv.x) * step(uv.x, 1.0f) * step(0.0f, uv.y) * step(uv.y, 1.0f);
    final_blend *= visible;
    float4 projection = tProjection.Sample(sSampler, uv);

    float4 final_color = lerp(0.0f.xxxx, projection, final_blend);
    return final_color;
}
