#include "common.hlsl"

//--------------------------------------------------------------------------------------
// Vertex Shader
v2f_t VertexFunction(vs_input_t input)
{
	v2f_t v2f = (v2f_t)0;

	// move the vertex through the spaces
	float4 local_pos = float4(input.position, 1.0f);               // passed in position is usually inferred to be "local position", ie, local to the object
	float4 world_pos = mul(MODEL, local_pos);               // world pos is the object moved to its place int he world by the model, not used yet
	float4 camera_pos = mul(VIEW, world_pos);
	float4 clip_pos = mul(PROJECTION, camera_pos);

	// normal is currently in model/local space
	float4 local_normal = float4(input.normal, 0);
	float4 world_normal = mul(MODEL, local_normal);

	float4 local_tangent = float4(input.tangent, 0);
	float4 world_tangent = mul(MODEL, local_tangent);

	float4 local_bitangent = float4(input.bitangent, 0);
	float4 world_bitangent = mul(MODEL, local_bitangent);

	world_normal = float4(cross(world_tangent.xyz, world_bitangent.xyz), 0);

	// tangent & bitangent

	v2f.position = clip_pos; // we want to output the clip position to raster (a perspective point)
	v2f.color = input.color;
	v2f.uv = input.uv;
	v2f.world_position = world_pos.xyz;
	v2f.world_normal = world_normal.xyz;
	v2f.world_tangent = world_tangent.xyz;
	v2f.world_bitangent = world_bitangent.xyz;

	return v2f;
}


float4 FragmentFunction(v2f_t input) : SV_Target0
{
    float3 normal = normalize(input.world_normal); 
    float3 tangent = normalize(input.world_tangent);
    float3 bitangent = normalize(input.world_bitangent); 
    
	float2 z_uv = frac(input.world_position.xy);
	z_uv.y = 1.0f - z_uv.y;

	float2 x_uv = frac(input.world_position.zy);
	// x_uv.x = 1.0f - x_uv.x; 
	x_uv.y = 1.0f - x_uv.y;

	float2 y_uv = frac(input.world_position.xz);

	float4 x_color = tDiffuse.Sample(sSampler, x_uv);
	float4 y_color = tDiffuse.Sample(sSampler, y_uv);
	float4 z_color = tDiffuse.Sample(sSampler, z_uv);

	float4 x_normalColor = tNormal.Sample(sSampler, x_uv);
	float4 y_normalColor = tNormal.Sample(sSampler, y_uv);
	float4 z_normalColor = tNormal.Sample(sSampler, z_uv);

	float3 x_normal = colorToNormalizedVector(x_normalColor);
	float3 y_normal = colorToNormalizedVector(y_normalColor);
	float3 z_normal = colorToNormalizedVector(z_normalColor);

	float3 weights = normal; 
    weights = abs(weights);
	float sum = weights.x + weights.y + weights.z; 
    weights /= sum; 

	z_normal.z *= sign(normal.z);

	float3x3 xtbn = float3x3(
		float3(0,0,-1),
		float3(0,1,0),
		float3(1,0,0)
	);
	x_normal = mul(x_normal, xtbn);

	x_normal.x *= sign(normal.x);


	float3x3 ytbn = float3x3(
		float3(1,0,0),
		float3(0,0,-1),
		float3(0,1,0)
	);
	y_normal = mul(y_normal, ytbn);
	y_normal.y *= sign(normal.y);


	float4 surface_color = weights.x * x_color + weights.y * y_color + weights.z * z_color;
	float3 surface_normal = weights.x * x_normal + weights.y * y_normal + weights.z * z_normal;
	surface_normal = normalize(surface_normal);
	//return normalizedVectorToColor(surface_normal);

	float3 final_color = ComputeLightingAt( input.world_position, surface_normal, surface_color.rgb);

	final_color = pow( final_color, INVERSE_GAMMA.xxx ); 

	float4 fColor = ApplyFog(float4(final_color.rgb, 1), length(input.world_position - CAMERA_POSITION));

	return float4(final_color.rgb, surface_color.a * fColor.a);
}